import React, { useEffect, useState } from 'react';
import { Panel, PanelGroup, PanelResizeHandle } from 'react-resizable-panels';
import Editor from './components/Editor';
import Sidebar from './components/Sidebar';
import AIPanel from './components/AIPanel';
import StatusBar from './components/StatusBar';
import CommandPalette from './components/CommandPalette';
import { useEditorStore } from './stores/editorStore';
import { useAIStore } from './stores/aiStore';
import { useKeyboardShortcuts } from './hooks/useKeyboardShortcuts';

function App() {
  const [showCommandPalette, setShowCommandPalette] = useState(false);
  const { isConnected, checkConnection } = useAIStore();
  const { currentFile, openFiles } = useEditorStore();

  // Setup keyboard shortcuts
  useKeyboardShortcuts({
    onToggleCommandPalette: () => setShowCommandPalette(true),
  });

  // Check AI service connection on startup
  useEffect(() => {
    checkConnection();
  }, [checkConnection]);

  return (
    <div className="h-screen bg-dark-900 text-dark-100 flex flex-col">
      {/* Main application layout */}
      <div className="flex-1 overflow-hidden">
        <PanelGroup direction="horizontal">
          {/* Sidebar */}
          <Panel defaultSize={20} minSize={15} maxSize={30}>
            <Sidebar />
          </Panel>
          
          <PanelResizeHandle className="w-1 bg-dark-700 hover:bg-primary-600 transition-colors" />
          
          {/* Main editor area */}
          <Panel defaultSize={60} minSize={40}>
            <div className="h-full flex flex-col">
              {/* Editor tabs */}
              {openFiles.length > 0 && (
                <div className="flex bg-dark-800 border-b border-dark-700">
                  {openFiles.map((file) => (
                    <div
                      key={file.path}
                      className={`px-4 py-2 border-r border-dark-700 cursor-pointer transition-colors ${
                        currentFile?.path === file.path
                          ? 'bg-dark-700 text-primary-400'
                          : 'hover:bg-dark-700'
                      }`}
                    >
                      <span className="text-sm">{file.name}</span>
                    </div>
                  ))}
                </div>
              )}
              
              {/* Editor */}
              <div className="flex-1">
                <Editor />
              </div>
            </div>
          </Panel>
          
          <PanelResizeHandle className="w-1 bg-dark-700 hover:bg-primary-600 transition-colors" />
          
          {/* AI Panel */}
          <Panel defaultSize={20} minSize={15} maxSize={40}>
            <AIPanel />
          </Panel>
        </PanelGroup>
      </div>
      
      {/* Status bar */}
      <StatusBar />
      
      {/* Command palette */}
      {showCommandPalette && (
        <CommandPalette
          isOpen={showCommandPalette}
          onClose={() => setShowCommandPalette(false)}
        />
      )}
      
      {/* Connection status indicator */}
      {!isConnected && (
        <div className="fixed top-4 right-4 bg-red-600 text-white px-4 py-2 rounded-lg shadow-lg animate-fade-in">
          <div className="flex items-center space-x-2">
            <div className="w-2 h-2 bg-red-300 rounded-full animate-pulse" />
            <span className="text-sm font-medium">AI Service Disconnected</span>
          </div>
        </div>
      )}
    </div>
  );
}

export default App; 