import { create } from 'zustand';

export interface AIMessage {
  id: string;
  type: 'user' | 'assistant' | 'system';
  content: string;
  timestamp: Date;
  command?: string;
  metadata?: {
    executionTime?: number;
    model?: string;
    language?: string;
    filePath?: string;
  };
}

export interface AIModel {
  name: string;
  size: string;
  description?: string;
  capabilities: string[];
  isAvailable: boolean;
}

interface AIState {
  // Connection state
  isConnected: boolean;
  isLoading: boolean;
  error: string | null;
  
  // Chat and messages
  messages: AIMessage[];
  currentModel: string;
  availableModels: AIModel[];
  
  // AI request state
  isProcessing: boolean;
  
  // Actions
  sendMessage: (content: string, command?: string) => Promise<void>;
  clearMessages: () => void;
  checkConnection: () => Promise<void>;
  loadModels: () => Promise<void>;
  setCurrentModel: (model: string) => void;
  explainCode: (code: string, language?: string) => Promise<string>;
  refactorCode: (code: string, language?: string) => Promise<string>;
  fixCode: (code: string, language?: string, error?: string) => Promise<string>;
  completeCode: (code: string, cursorPosition: number, language?: string) => Promise<string>;
  setError: (error: string | null) => void;
}

const API_BASE_URL = 'http://localhost:8000/api';

export const useAIStore = create<AIState>((set, get) => ({
  isConnected: false,
  isLoading: false,
  error: null,
  messages: [],
  currentModel: 'codellama:7b-code',
  availableModels: [],
  isProcessing: false,

  sendMessage: async (content, command) => {
    const messageId = Date.now().toString();
    const userMessage: AIMessage = {
      id: messageId,
      type: 'user',
      content,
      timestamp: new Date(),
      command,
    };

    // Add user message immediately
    set(state => ({
      messages: [...state.messages, userMessage],
      isProcessing: true,
      error: null,
    }));

    try {
      const response = await fetch(`${API_BASE_URL}/ai/process`, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify({
          command: command || 'explain',
          code: content,
          model: get().currentModel,
        }),
      });

      if (!response.ok) {
        throw new Error(`HTTP error! status: ${response.status}`);
      }

      const data = await response.json();

      if (data.success) {
        const assistantMessage: AIMessage = {
          id: `${messageId}_response`,
          type: 'assistant',
          content: data.result,
          timestamp: new Date(),
          metadata: {
            executionTime: data.execution_time,
            model: data.model_used,
          },
        };

        set(state => ({
          messages: [...state.messages, assistantMessage],
          isProcessing: false,
        }));
      } else {
        throw new Error(data.result || 'Unknown error');
      }
    } catch (error) {
      const errorMessage: AIMessage = {
        id: `${messageId}_error`,
        type: 'system',
        content: `Error: ${error instanceof Error ? error.message : 'Unknown error'}`,
        timestamp: new Date(),
      };

      set(state => ({
        messages: [...state.messages, errorMessage],
        isProcessing: false,
        error: error instanceof Error ? error.message : 'Unknown error',
      }));
    }
  },

  clearMessages: () => set({ messages: [] }),

  checkConnection: async () => {
    try {
      set({ isLoading: true, error: null });
      
      const response = await fetch(`${API_BASE_URL}/ai/health`);
      const data = await response.json();
      
      set({
        isConnected: response.ok && data.ollama_available,
        isLoading: false,
      });
    } catch (error) {
      set({
        isConnected: false,
        isLoading: false,
        error: 'Failed to connect to AI service',
      });
    }
  },

  loadModels: async () => {
    try {
      set({ isLoading: true });
      
      const response = await fetch(`${API_BASE_URL}/models/list`);
      if (!response.ok) {
        throw new Error('Failed to load models');
      }
      
      const data = await response.json();
      
      set({
        availableModels: data.models,
        currentModel: data.current_model || get().currentModel,
        isLoading: false,
      });
    } catch (error) {
      set({
        error: 'Failed to load models',
        isLoading: false,
      });
    }
  },

  setCurrentModel: (model) => set({ currentModel: model }),

  explainCode: async (code, language) => {
    try {
      set({ isProcessing: true, error: null });
      
      const response = await fetch(`${API_BASE_URL}/ai/explain`, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify({
          code,
          language,
          detail_level: 'medium',
        }),
      });

      const data = await response.json();
      
      if (data.success) {
        set({ isProcessing: false });
        return data.result;
      } else {
        throw new Error(data.result);
      }
    } catch (error) {
      set({
        error: error instanceof Error ? error.message : 'Unknown error',
        isProcessing: false,
      });
      throw error;
    }
  },

  refactorCode: async (code, language) => {
    try {
      set({ isProcessing: true, error: null });
      
      const response = await fetch(`${API_BASE_URL}/ai/refactor`, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify({
          code,
          language,
          refactor_type: 'general',
        }),
      });

      const data = await response.json();
      
      if (data.success) {
        set({ isProcessing: false });
        return data.result;
      } else {
        throw new Error(data.result);
      }
    } catch (error) {
      set({
        error: error instanceof Error ? error.message : 'Unknown error',
        isProcessing: false,
      });
      throw error;
    }
  },

  fixCode: async (code, language, errorMessage) => {
    try {
      set({ isProcessing: true, error: null });
      
      const response = await fetch(`${API_BASE_URL}/ai/fix`, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify({
          code,
          language,
          error_message: errorMessage,
        }),
      });

      const data = await response.json();
      
      if (data.success) {
        set({ isProcessing: false });
        return data.result;
      } else {
        throw new Error(data.result);
      }
    } catch (error) {
      set({
        error: error instanceof Error ? error.message : 'Unknown error',
        isProcessing: false,
      });
      throw error;
    }
  },

  completeCode: async (code, cursorPosition, language) => {
    try {
      set({ isProcessing: true, error: null });
      
      const response = await fetch(`${API_BASE_URL}/ai/complete`, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify({
          code,
          cursor_position: cursorPosition,
          language,
          max_tokens: 50,
        }),
      });

      const data = await response.json();
      
      if (data.success && data.completions.length > 0) {
        set({ isProcessing: false });
        return data.completions[0];
      } else {
        throw new Error('No completions available');
      }
    } catch (error) {
      set({
        error: error instanceof Error ? error.message : 'Unknown error',
        isProcessing: false,
      });
      throw error;
    }
  },

  setError: (error) => set({ error }),
})); 