import { create } from 'zustand';

export interface FileInfo {
  path: string;
  name: string;
  content: string;
  language: string;
  isDirty: boolean;
  cursorPosition?: {
    line: number;
    column: number;
  };
}

interface EditorState {
  // File management
  openFiles: FileInfo[];
  currentFile: FileInfo | null;
  
  // Editor state
  isLoading: boolean;
  error: string | null;
  
  // Actions
  openFile: (file: Omit<FileInfo, 'isDirty'>) => void;
  closeFile: (path: string) => void;
  setCurrentFile: (path: string) => void;
  updateFileContent: (path: string, content: string) => void;
  saveFile: (path: string) => Promise<void>;
  saveCurrentFile: () => Promise<void>;
  setCursorPosition: (path: string, line: number, column: number) => void;
  setError: (error: string | null) => void;
  setLoading: (loading: boolean) => void;
}

export const useEditorStore = create<EditorState>((set, get) => ({
  openFiles: [],
  currentFile: null,
  isLoading: false,
  error: null,

  openFile: (file) => {
    const { openFiles } = get();
    
    // Check if file is already open
    const existingFile = openFiles.find(f => f.path === file.path);
    if (existingFile) {
      set({ currentFile: existingFile });
      return;
    }
    
    // Add new file
    const newFile: FileInfo = {
      ...file,
      isDirty: false,
    };
    
    set({
      openFiles: [...openFiles, newFile],
      currentFile: newFile,
    });
  },

  closeFile: (path) => {
    const { openFiles, currentFile } = get();
    const updatedFiles = openFiles.filter(f => f.path !== path);
    
    // If closing current file, select another one
    let newCurrentFile = currentFile;
    if (currentFile?.path === path) {
      newCurrentFile = updatedFiles.length > 0 ? updatedFiles[0] : null;
    }
    
    set({
      openFiles: updatedFiles,
      currentFile: newCurrentFile,
    });
  },

  setCurrentFile: (path) => {
    const { openFiles } = get();
    const file = openFiles.find(f => f.path === path);
    if (file) {
      set({ currentFile: file });
    }
  },

  updateFileContent: (path, content) => {
    const { openFiles, currentFile } = get();
    
    const updatedFiles = openFiles.map(file => {
      if (file.path === path) {
        const updatedFile = {
          ...file,
          content,
          isDirty: content !== file.content,
        };
        return updatedFile;
      }
      return file;
    });
    
    // Update current file if it matches
    const updatedCurrentFile = currentFile?.path === path 
      ? updatedFiles.find(f => f.path === path) 
      : currentFile;
    
    set({
      openFiles: updatedFiles,
      currentFile: updatedCurrentFile || null,
    });
  },

  saveFile: async (path) => {
    const { openFiles } = get();
    const file = openFiles.find(f => f.path === path);
    
    if (!file) return;
    
    try {
      set({ isLoading: true, error: null });
      
      // Use Tauri's file writing capability
      const { writeFile } = await import('@tauri-apps/api/fs');
      await writeFile(path, file.content);
      
      // Mark file as saved
      const updatedFiles = openFiles.map(f => 
        f.path === path ? { ...f, isDirty: false } : f
      );
      
      set({
        openFiles: updatedFiles,
        currentFile: updatedFiles.find(f => f.path === path) || null,
        isLoading: false,
      });
      
    } catch (error) {
      set({
        error: `Failed to save file: ${error}`,
        isLoading: false,
      });
    }
  },

  saveCurrentFile: async () => {
    const { currentFile, saveFile } = get();
    if (currentFile) {
      await saveFile(currentFile.path);
    }
  },

  setCursorPosition: (path, line, column) => {
    const { openFiles, currentFile } = get();
    
    const updatedFiles = openFiles.map(file => {
      if (file.path === path) {
        return {
          ...file,
          cursorPosition: { line, column },
        };
      }
      return file;
    });
    
    const updatedCurrentFile = currentFile?.path === path 
      ? updatedFiles.find(f => f.path === path) 
      : currentFile;
    
    set({
      openFiles: updatedFiles,
      currentFile: updatedCurrentFile || null,
    });
  },

  setError: (error) => set({ error }),
  setLoading: (loading) => set({ isLoading: loading }),
})); 